%This code runs experiments in Octave. The code is a modification of exemplary code from COCO (http://coco.gforge.inria.fr/)
%modified by Rafał Biedrzycki

more off; % to get immediate output in Octave

source("cmaes.m")

%%%%%%%%%%%%%%%%%%%%%%%%%
% Experiment Parameters %
%%%%%%%%%%%%%%%%%%%%%%%%%
BUDGET_MULTIPLIER = 10000000; % algorithm runs for BUDGET_MULTIPLIER*dimension funevals
NUM_OF_INDEPENDENT_RESTARTS = 1e9; % max. number of independent algorithm
% restarts; if >0, make sure that the
% algorithm is not always doing the same thing
% in each run (which is typically trivial for
% randomized algorithms)

%%%%%%%%%%%%%%%%%%%%%%%%%
% Prepare Experiment    %
%%%%%%%%%%%%%%%%%%%%%%%%%

% choose a test suite and a matching logger, for
% example one of the following:
%
% bbob              24 unconstrained noiseless single-objective functions
% bbob-biobj        55 unconstrained noiseless bi-objective functions
% bbob-biobj-ext    92 unconstrained noiseless bi-objective functions
% bbob-largescale   24 unconstrained noiseless single-objective functions in large dimensions
% bbob-constrained  48 constrained noiseless single-objective functions
%
suite_name = 'bbob';
observer_name = 'bbob';
observer_options = strcat('result_folder: Matlab', ...
    suite_name, ...
    [' algorithm_name: MatCma'...
    ' algorithm_info: Matlab CMA_ES ']);

% initialize suite and observer with default options,
% to change the default, see 
% http://numbbo.github.io/coco-doc/C/#suite-parameters and
% http://numbbo.github.io/coco-doc/C/#observer-parameters
% for details.
%suite = cocoSuite(suite_name, 'year: 2009', '');
suite = cocoSuite(suite_name, 'year: 2009', 'dimensions: 5');
observer = cocoObserver(observer_name, observer_options);

% set log level depending on how much output you want to see, e.g. 'warning'
% for fewer output than 'info'.
cocoSetLogLevel('warning');

% keep track of problem dimension and #funevals to print timing information:
printeddim = 0;
doneEvalsAfter = 0; % summed function evaluations for a single problem
doneEvalsTotal = 0; % summed function evaluations per dimension
printstring = '\n'; % store strings to be printed until experiment is finished


function res=fn(x, problem)
  %res=benchmark_func(x', f_nr);
  %disp(x)
  res=cocoEvaluateFunction(problem, x');
endfunction


%%%%%%%%%%%%%%%%%%%%%%%%%
% Run Experiment        %
%%%%%%%%%%%%%%%%%%%%%%%%%
while true
    % get next problem and dimension from the chosen suite:
    problem = cocoSuiteGetNextProblem(suite, observer);
    if ~cocoProblemIsValid(problem)
        break;
    end
    dimension = cocoProblemGetDimension(problem);
    
    % printing
    if printeddim < dimension
      if printeddim > 1
        elapsedtime = toc;
        printstring = strcat(printstring, ...
            sprintf('   COCO TIMING: dimension %d finished in %e seconds/evaluation\n', ...
            printeddim, elapsedtime/double(doneEvalsTotal)));
        tic;
      end
      doneEvalsTotal = 0;
      printeddim = dimension;
      tic;
    end  
    
    % restart functionality: do at most NUM_OF_INDEPENDENT_RESTARTS+1
    % independent runs until budget is used:
    startSol = cocoProblemGetInitialSolution(problem)';
    %disp(startSol)
    i = -1; % count number of independent restarts
    while (BUDGET_MULTIPLIER*dimension > (cocoProblemGetEvaluations(problem) + cocoProblemGetEvaluationsConstraints(problem)) )
        i = i+1;
        if (i > 0)
            fprintf('INFO: algorithm restarted\n');
        end
        doneEvalsBefore = cocoProblemGetEvaluations(problem) + ...
                          cocoProblemGetEvaluationsConstraints(problem);
        
        opts=cmaes;
        opts.MaxFunEvals = BUDGET_MULTIPLIER*dimension - doneEvalsBefore;
        opts.MaxIter=opts.MaxFunEvals;%zbyt duzo ale co z tego
        opts.LogPlot = 'off';
        opts.DispModulo=0;
        opts.LBounds = cocoProblemGetSmallestValuesOfInterest(problem)(1);
        opts.UBounds = cocoProblemGetLargestValuesOfInterest(problem)(1);
        SIGMA=0.3*(opts.UBounds - opts.LBounds);
  

        cmaes("fn", startSol, SIGMA, opts, problem );
        
        startSol = opts.LBounds+(opts.UBounds-opts.LBounds)*(rand(dimension,1)+rand(dimension,1))/2;
        
        % check whether things went wrong or whether experiment is over:
        doneEvalsAfter = cocoProblemGetEvaluations(problem) + ...
                         cocoProblemGetEvaluationsConstraints(problem);
        if cocoProblemFinalTargetHit(problem) == 1 ||...
                doneEvalsAfter >= BUDGET_MULTIPLIER * dimension
            break;
        end
        if (doneEvalsAfter == doneEvalsBefore)
            fprintf('WARNING: Budget has not been exhausted (%d/%d evaluations done)!\n', ....
                doneEvalsBefore, BUDGET_MULTIPLIER * dimension);
            break;
        end
        if (doneEvalsAfter < doneEvalsBefore)
            fprintf('ERROR: Something weird happened here which should not happen: f-evaluations decreased');
        end
        if (i >= NUM_OF_INDEPENDENT_RESTARTS)
            break;
        end
    end
    
    doneEvalsTotal = doneEvalsTotal + doneEvalsAfter;
end

elapsedtime = toc;
printstring = strcat(printstring, ...
    sprintf('   COCO TIMING: dimension %d finished in %e seconds/evaluation\n', ...
    printeddim, elapsedtime/double(doneEvalsTotal)));
fprintf(printstring);

cocoObserverFree(observer);
cocoSuiteFree(suite);
