#!/usr/bin/env python
#This code runs experiments using Python implementation (cma) configured to use weighted quadratic penalty for bound constraint handling. 
#The code is a modification of exemplary code from COCO (http://coco.gforge.inria.fr/) aimed at running CMA-ES (http://cma.gforge.inria.fr/cmaesintro.html).
#modified by Rafał Biedrzycki

from __future__ import division, print_function
import cocoex
import cma
from numpy.random import rand  # for randomised restarts
from cocoex import Suite, Observer, log_level
import numpy as np

### input
suite_name = "bbob"
output_folder = "PySqPen"
budget_multiplier = 1e7  

### prepare
suite_instance = "year:2009"
suite_options = "dimensions: 5"  # "dimensions: 2,3,5,10,20 "  # if 40 is not desired
suite = Suite(suite_name, suite_instance, suite_options)
observer = cocoex.Observer(suite_name, "result_folder: " + output_folder)
minimal_print = cocoex.utilities.MiniPrint()

### go
for problem in suite:  # this loop will take several minutes or longer
    print(problem)
    problem.observe_with(observer)  # generates the data for cocopp post-processing
    x0 = problem.initial_solution
    SIGMA = 0.3*(problem.upper_bounds[0] - problem.lower_bounds[0])
    # apply restarts while neither the problem is solved nor the budget is exhausted
    restartNmbr=1
    while (problem.evaluations < problem.dimension * budget_multiplier and not problem.final_target_hit):
        budget_left = problem.dimension * budget_multiplier - problem.evaluations
        default_popsize = 4 + int(3 * np.log(problem.dimension))
        max_iters=int(problem.dimension * budget_multiplier/default_popsize)
        cma.fmin( problem, x0, SIGMA, options={ 'verb_disp':0, 'verb_log':0, 'verb_time':False, 'verbose':-9, 'maxfevals': budget_left, 'maxiter':max_iters, 'bounds': [problem.lower_bounds[0], problem.upper_bounds[0]],'BoundaryHandler': cma.BoundPenalty} )
        x0 = problem.initial_solution_proposal(restartNmbr)
        restartNmbr=restartNmbr+1
    minimal_print(problem, final=problem.index == len(suite) - 1)


